import { Component, OnInit, Input,AfterViewInit } from '@angular/core';
import { IColorMap, IColor, IColorEntry,ILayer, IRenderer } from '../../models/color.map';


@Component({
  selector: 'fm-map-layer-legend',
  templateUrl: './legend.component.html',
  styleUrls: ['./legend.component.scss']
})
export class LegendComponent implements OnInit,AfterViewInit {

  constructor()  {

  }

  ngOnInit() {
  }

  ngAfterViewInit() {
  }

  @Input()
  layer: ILayer;

  @Input()
  legend: string;

  @Input()
  histogram: string;

  @Input()
  showTitle: boolean = true;

  @Input()
  histogramenabled: boolean;

  @Input()
  legendunit: string;

  @Input()
  histogramunit: string;


  onClickHistoGram(): void {
    this.histogramenabled = !this.histogramenabled;
  }




  public getHex(color: IColor): string {
    return '#' + this.componentToHex(color.red) + this.componentToHex(color.green) + this.componentToHex(color.blue);
  }

  private componentToHex(c: number): string {
    const hex = c.toString(16);
    return hex.length === 1 ? `0${hex}` : hex;
  }

  public getPart(renderer: IRenderer, index: number): string {
    let max = renderer.band.histogram.entries.reduce((max, entry) => entry.freqency > max ? entry.freqency : max, 0);
    let scale = 75 / max;
    let part = (renderer.band.histogram.entries[index].freqency * scale) + "%";

    return part;
  }

  public getLabel(renderer: IRenderer, index: number): string {
    let scale = 100 / renderer.band.histogram.entries.reduce((sum, entry) => sum + entry.freqency, 0);
    let percent = renderer.band.histogram.entries[index].freqency * scale;
    return percent < 0.1 ? "" : percent.toFixed(1) + " %";
  }



  showHistogram(): boolean {
    return this.histogramenabled && this.layer.renderer.band.histogram.entries && this.layer.renderer.band.histogram.entries.length > 0 && this.layer.renderer.colorMap.colormapType == "minmax";
  }
}


